"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _getListsError = _interopRequireDefault(require("../../utils/errors/getListsError"));

var _parseListItems = _interopRequireDefault(require("../../utils/lists/parseListItems"));

var _getListsOptions2 = _interopRequireDefault(require("../../utils/options/getListsOptions"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

/**
 * @name getListItems
 * @description Returns information about items in the list based on the specified query.
 * @param {Object} options
 * @param {String} options.listName - The list display name.
 * @param {String} [options.viewName=""] - The view GUID.
 * @param {String} [options.CAMLViewFields="<ViewFields Properties='True' />"] - The view fields as an XML string
 * @param {String} [options.CAMLViewFields="<ViewFields Properties='True' />"] - The view fields as an XML string
 * @param {String} [options.CAMLQuery="<Query></Query>"] - The CAML query as an XML string
 * @param {String} [options.CAMLQueryOptions="<QueryOptions></QueryOptions>"] - Additional query options as an XML string.
 * @param {Number} [options.CAMLRowLimit=0] - The number of rows to retrieve
 * @param {Number} [options.timeout=10] - The timeout (pause) between iterations when parsing items.
 * @param {Boolean} [options.viewColumns=null] - Only columns that are selectable in views will be returned (only if parsing is true)
 * @param {Boolean} [options.async=true] - Synchronisation Type
 * @param {Boolean} [options.parseItems=false] - XML will be parsed with items return
 * @returns {Promise} Returns a Promise XML result.
 * @example - // Query list on same site
 * getListItems({
 *    listName: "Announcements"
 * });
 * @example - // Query list on another site
 * getListItems({
 *  listName: "Announcements",
 *  webURL: "http://objectpoint.com/sites/defence"
 * });
 * @example - // Advanced query
 * getListItems({
 *  listName: "Announcements",
 *  webURL: "http://objectpoint.com/sites/defence",
 *  CAMLQuery: '<Query><Where><Eq><FieldRef Name="FirstName" /><Value Type="Text">Mapero</Value></Eq></Where></Query>',
 *  CAMLViewFields: '<ViewFields><FieldRef Name="Title" /></ViewFields>'
 *  parseItems: true
 * });
 */
var getListItems = function getListItems(options) {
  return new Promise(function (resolve, reject) {
    var _getListsOptions = (0, _getListsOptions2["default"])(options),
        webURL = _getListsOptions.webURL,
        async = _getListsOptions.async,
        listName = _getListsOptions.listName,
        CAMLViewFields = _getListsOptions.CAMLViewFields,
        viewName = _getListsOptions.viewName,
        CAMLQuery = _getListsOptions.CAMLQuery,
        CAMLQueryOptions = _getListsOptions.CAMLQueryOptions,
        CAMLRowLimit = _getListsOptions.CAMLRowLimit,
        parseItems = _getListsOptions.parseItems,
        timeout = _getListsOptions.timeout,
        viewColumns = _getListsOptions.viewColumns; // Create payload


    var payload = {
      operation: "GetListItems",
      webURL: webURL,
      async: async,
      listName: listName,
      CAMLViewFields: CAMLViewFields,
      CAMLQueryOptions: CAMLQueryOptions,
      CAMLRowLimit: CAMLRowLimit,
      completefunc: function completefunc(xData, status) {
        if (status === "success") {
          if (parseItems) {
            var rows = $(xData.responseXML).SPFilterNode("z:row").toArray();
            (0, _parseListItems["default"])({
              rows: rows,
              CAMLViewFields: CAMLViewFields,
              timeout: timeout,
              viewColumns: viewColumns
            }).then(function (items) {
              resolve(items);
            });
          } else {
            resolve(xData.responseXML);
          }
        } else {
          reject((0, _getListsError["default"])(xData));
        }
      }
    }; // If CAMLQuery is truthy

    if (CAMLQuery) {
      // Add to payload
      payload.CAMLQuery = CAMLQuery;
    } // If viewName is truthy
    else if (viewName) {
        payload.viewName = viewName;
      } else {
        // Warn that default view will be received
        console.warn("No CAMLQuery or viewName was provided. The items returned may be from the default view.");
      } // Send request


    $().SPServices(payload);
  });
};

var _default = getListItems;
exports["default"] = _default;