/**
 * @name addList
 * @description Creates a list in the current site based on the specified name, description, and list template ID.
 * @param {Object} payload The configuration payload object
 * @param {String} payload.listName The list static name
 * @param {Number|String} payload.templateID The list template ID
 * @param {String} payload.description A short description of the list
 * @param {String} [payload.webURL=window.$().SPServices.defaults.webURL] The web site url
 * @param {Boolean} [payload.parse=false] If true, the response is returned as an object
 * @returns {Promise<Object>} Returns an XML object or if parsed, a list object
 */
const addList = ({
  async = true,
  description = "",
  listName,
  templateID = 100,
  webURL = window.$().SPServices.defaults.webURL,
  parse = false,
} = {}) =>
  new Promise((resolve, reject) => {
    // if listName is not defined.
    if (!listName) {
      // Reject with error and end function
      return reject(new Error("listName not defined. Unable to create list."));
    }

    $().SPServices({
      operation: "AddList",
      async,
      description,
      listName,
      templateID,
      webURL,
      completefunc: function (xData, status) {
        // Get XML Response
        const xml = xData.responseXML;

        // Check for error text
        const errorText = (
          (xml.querySelector("errorstring") || {}).textContent || ""
        ).trim();

        // If error is found
        if (errorText || status === "error") {
          // Create error object
          const error = new Error(errorText || "Unable to create the list.");
          error.status = xData.status;
          error.statusText = xData.statusText;
          // Reject with error
          return reject(error);
        }

        // Get xml response
        const response = xml.querySelector("List");

        // If parse is false, resolve response
        if (!parse) return resolve(response);

        // Create list object
        let list = [...response.attributes].reduce(
          (object, { name, value }) => {
            object[name] = value || "";
            return object;
          },
          {}
        );

        // Get fields
        list.fields = [...response.querySelectorAll("Field")].map((field) => {
          return [...field.attributes].reduce((object, { name, value }) => {
            object[name] = value || "";
            return object;
          }, {});
        });

        // Get regional settings
        list.regionalSettings = [
          ...response.querySelector("RegionalSettings").childNodes,
        ].reduce((object, { nodeName, textContent }) => {
          object[nodeName] = textContent || "";
          return object;
        }, {});

        // Get server settings
        list.serverSettings = [
          ...response.querySelector("ServerSettings").childNodes,
        ].reduce((object, { nodeName, textContent }) => {
          object[nodeName] = textContent || "";
          return object;
        }, {});

        // Resolve parsed list
        return resolve(list);
      },
    });
  });

export default addList;
