import getListsError from "../../utils/errors/getListsError";
import parseListItems from "../../utils/lists/parseListItems";
import getListsOptions from "../../utils/options/getListsOptions";

/**
 * @name getListItems
 * @description Returns information about items in the list based on the specified query.
 * @param {Object} options
 * @param {String} options.listName - The list display name.
 * @param {String} [options.viewName=""] - The view GUID.
 * @param {String} [options.CAMLViewFields="<ViewFields Properties='True' />"] - The view fields as an XML string
 * @param {String} [options.CAMLViewFields="<ViewFields Properties='True' />"] - The view fields as an XML string
 * @param {String} [options.CAMLQuery="<Query></Query>"] - The CAML query as an XML string
 * @param {String} [options.CAMLQueryOptions="<QueryOptions></QueryOptions>"] - Additional query options as an XML string.
 * @param {Number} [options.CAMLRowLimit=0] - The number of rows to retrieve
 * @param {Number} [options.timeout=10] - The timeout (pause) between iterations when parsing items.
 * @param {Boolean} [options.viewColumns=null] - Only columns that are selectable in views will be returned (only if parsing is true)
 * @param {Boolean} [options.async=true] - Synchronisation Type
 * @param {Boolean} [options.parseItems=false] - XML will be parsed with items return
 * @returns {Promise} Returns a Promise XML result.
 * @example - // Query list on same site
 * getListItems({
 *    listName: "Announcements"
 * });
 * @example - // Query list on another site
 * getListItems({
 *  listName: "Announcements",
 *  webURL: "http://objectpoint.com/sites/defence"
 * });
 * @example - // Advanced query
 * getListItems({
 *  listName: "Announcements",
 *  webURL: "http://objectpoint.com/sites/defence",
 *  CAMLQuery: '<Query><Where><Eq><FieldRef Name="FirstName" /><Value Type="Text">Mapero</Value></Eq></Where></Query>',
 *  CAMLViewFields: '<ViewFields><FieldRef Name="Title" /></ViewFields>'
 *  parseItems: true
 * });
 */
const getListItems = (options) => {
  return new Promise((resolve, reject) => {
    const {
      webURL,
      async,
      listName,
      CAMLViewFields,
      viewName,
      CAMLQuery,
      CAMLQueryOptions,
      CAMLRowLimit,
      parseItems,
      timeout,
      viewColumns,
    } = getListsOptions(options);

    // Create payload
    let payload = {
      operation: "GetListItems",
      webURL,
      async,
      listName,
      CAMLViewFields,
      CAMLQueryOptions,
      CAMLRowLimit,
      completefunc: function (xData, status) {
        if (status === "success") {
          if (parseItems) {
            const rows = $(xData.responseXML).SPFilterNode("z:row").toArray();
            parseListItems({
              rows,
              CAMLViewFields,
              timeout,
              viewColumns,
            }).then((items) => {
              resolve(items);
            });
          } else {
            resolve(xData.responseXML);
          }
        } else {
          reject(getListsError(xData));
        }
      },
    };

    // If CAMLQuery is truthy
    if (CAMLQuery) {
      // Add to payload
      payload.CAMLQuery = CAMLQuery;
    }
    // If viewName is truthy
    else if (viewName) {
      payload.viewName = viewName;
    } else {
      // Warn that default view will be received
      console.warn(
        "No CAMLQuery or viewName was provided. The items returned may be from the default view."
      );
    }

    // Send request
    $().SPServices(payload);
  });
};

export default getListItems;
