// API
import getList from "./getList";

// Utils
import escapeHtml from "../../utils/general/escapeHtml";

/**
 * @name updateList
 * @description Creates a list in the current site based on the specified name, description, and list template ID.
 * @param {Object} payload The configuration payload object
 * @param {String} payload.listName The list static name
 * @param {String} [payload.webURL=window.$().SPServices.defaults.webURL] The web site url
 * @param {Boolean} [payload.parse=false] If true, the response is returned as an object
 * @param {Object} [payload.listProperties=null] The list static name
 * @param {String} [payload.listProperties.description] A string that contains the description for the list.
 * @param {String} [payload.listProperties.direction] A string that contains LTR if the reading order is left-to-right, RTL if it is right-to-left, or None.
 * @param {Boolean} [payload.listProperties.enableAttachments] true to enable attachments to items in the list. Does not apply to document libraries.
 * @param {Boolean} [payload.listProperties.enableVersioning] true to enable versioning for the list.
 * @param {Boolean} [payload.listProperties.enforceDataValidation] ?.
 * @param {Boolean} [payload.listProperties.hidden] true to hide the list so that it does not appear on the Documents and Lists page, Quick Launch bar, Modify Site Content page, or Add Column page as an option for lookup fields.
 * @param {Boolean} [payload.listProperties.ordered] true to specify that the option to allow users to reorder items in the list is available on the Edit View page for the list.
 * @param {Boolean} [payload.listProperties.preserveEmptyValues] ?.
 * @param {Boolean} [payload.listProperties.showUser] ?.
 * @param {Boolean} [payload.listProperties.strictTypeCoercion] ?.
 * @param {Boolean} [payload.listProperties.enableMinorVersion] true enables minor versioning (only avaialble for certain list types).
 * @param {Boolean} [payload.listProperties.requireCheckout] true enables checkout for document libraries only.
 * @param {Boolean} [payload.listProperties.allowMultiResponses] true enables multiple responses for survey lists only.
 * @param {String} [payload.listProperties.Title] A string that contains the title of the list.
 * @param {Object[]} [payload.newFields] An array containing field objects to be created
 * @param {String} payload.newFields[].name The field static name. This is required to locate the field.
 * @param {String} payload.newFields[].type The field type
 * @param {Boolean} [payload.newFields[].displayName] The field display name.
 * @param {Boolean} [payload.newFields[].enforceUniqueValues] If true, unique values are enforced (column must be indexed first)
 * @param {String[]} [payload.newFields[].choices] An array of choices (For choice field types only)
 * @param {Boolean} [payload.newFields[].indexed] If true, the column will be indexed
 * @param {Boolean} [payload.newFields[].readOnly] If true, the column will be read only
 * @param {Boolean} [payload.newFields[].required] If true, the column will be required
 * @param {Number} [payload.newFields[].max] The maximum value for number fields only
 * @param {Number} [payload.newFields[].min] The minimum value for number fields only
 * @param {Number} [payload.newFields[].maxLength] The maximum length for text fields only
 * @param {Object[]} [payload.updateFields] An array containing field objects to be updated
 * @param {String} payload.updateFields[].name The field static name. This is required to locate the field.
 * @param {String} payload.updateFields[].type The field type
 * @param {Boolean} [payload.updateFields[].displayName] The field display name.
 * @param {Boolean} [payload.updateFields[].enforceUniqueValues] If true, unique values are enforced (column must be indexed first)
 * @param {String[]} [payload.updateFields[].choices] An array of choices (For choice field types only)
 * @param {Boolean} [payload.updateFields[].indexed] If true, the column will be indexed
 * @param {Boolean} [payload.updateFields[].readOnly] If true, the column will be read only
 * @param {Boolean} [payload.updateFields[].required] If true, the column will be required
 * @param {Number} [payload.updateFields[].max] The maximum value for number fields only
 * @param {Number} [payload.updateFields[].min] The minimum value for number fields only
 * @param {Number} [payload.updateFields[].maxLength] The maximum length for text fields only
 * @param {Object[]} [payload.deleteFields] An array containing field objects to be deleted
 * @param {String} payload.deleteFields[].name The field static name. This is required to locate the field.
 * @returns {Promise<Object>} Returns an XML object or if parsed, a list object
 */

const updateList = ({
  async = true,
  listName,
  listProperties = null,
  newFields = [],
  updateFields = [],
  deleteFields = [],
  webURL = window.$().SPServices.defaults.webURL,
  parse = false,
} = {}) =>
  new Promise((resolve, reject) => {
    // Allowed keys for fields
    const allowedKeys = [
      "CHOICES",
      "DISPLAYNAME",
      "ENFORCEUNIQUEVALUES",
      "HIDDEN",
      "INDEXED",
      "MAX",
      "MAXLENGTH",
      "MIN",
      "NAME",
      "READONLY",
      "REQUIRED",
      "TYPE",
    ];

    // if listName is not defined.
    if (!listName) {
      // Reject with error and end function
      return reject(
        new Error("listName not defined. Unable to update the list.")
      );
    }

    // Create the request payload
    const payload = {
      operation: "UpdateList",
      async,
      listName,
      webURL,
    };

    // If list listProperties are defined
    if (listProperties) {
      // Add listProperties to the payload as an xml string
      payload.listProperties =
        "<List " +
        Object.keys(listProperties)
          .map((key) => {
            // Get value
            let value = listProperties[key];

            // Get attribute name
            let name = key[0].toUpperCase() + key.slice(1);

            // If boolean, convert to string
            if (typeof value === "boolean") {
              value = value ? "TRUE" : "FALSE";
            }

            // Return string
            return `${name}="${value}"`;
          })
          .join(" ") +
        " />";
    }

    // Create Methods
    if (newFields.length > 0) {
      payload.newFields = createFieldMethods(newFields);
    }

    if (updateFields.length > 0) {
      payload.updateFields = createFieldMethods(updateFields);
    }
    if (deleteFields.length > 0) {
      payload.deleteFields = createFieldMethods(deleteFields);
    }

    // Send the request
    $().SPServices({
      ...payload,
      completefunc: function (xData, status) {
        // Get XML Response
        const xml = xData.responseXML;

        // Check for error text
        const errorText = (
          (
            xml.querySelector("errorstring") ||
            xml.querySelector("ErrorText") ||
            {}
          ).textContent || ""
        ).trim();

        // Get the new list name (if being updated in list properties)
        const newListName = listProperties
          ? listProperties.title || listProperties.Title || ""
          : "";

        // If error is found
        if (errorText || status === "error") {
          // If the list is being update an error is always thrown (SharePoint bug)
          if (newListName) {
            // To get a valid response, get the list data, by the new list name.
            return getList({ webURL, listName: newListName, parse }).then(
              (res) => {
                return resolve(res);
              }
            );
          } else {
            // Create error object
            const error = new Error(errorText || "Unable to update list.");
            error.status = xData.status;
            error.statusText = xData.statusText;
            // Reject with error
            return reject(error);
          }
        }

        // Query the list to get all new properties and fields
        getList({ webURL, listName, parse }).then((res) => {
          // Return the resolved promise and response
          return resolve(res);
        });
      },
    });

    /**
     * @name createFieldMethods
     * @description Creates field methods from an array of field objects
     * @param {Object[]} fields
     * @returns {String} Returns a XML string.
     */
    function createFieldMethods(fields) {
      return `<Fields>
      ${fields
        .map((field, index) => {
          // Open method tag and assign ID
          let method = `<Method ID="${index + 1}">`;

          // If the field is already a string
          if (typeof field === "string") {
            // Update the method
            method += field;
          } else {
            // Open field
            method += "<Field";

            // Iterate through all keys in field
            for (const key in field) {
              // Uppercase KEY
              const KEY = key.toUpperCase();

              // If key is not equal to choices
              if (allowedKeys.includes(KEY) && KEY !== "CHOICES") {
                // Convert key to start with uppercase
                let name = key[0].toUpperCase() + key.slice(1);

                // Get value
                let value = field[key];

                // If boolean, convert to string
                if (typeof value === "boolean") {
                  value = value ? "TRUE" : "FALSE";
                }

                // Add to method
                method += ` ${name}="${value}"`;
              }
            }
            // Close opening field tag
            method += ">";

            // Get choices
            const choices = field.choices || field.Choices || field.CHOICES;

            // If choices is defined
            if (choices) {
              // Update choices in method
              method += `<CHOICES>
              ${choices
                .map((choice) => `<CHOICE>${escapeHtml(choice)}</CHOICE>`)
                .join("")}
              </CHOICES>`;
            }

            // Close the field tag
            method += "</Field>";
          }

          // Close the method tag
          method += "</Method>";

          // Return the method
          return method;
        })
        .join("")}
      </Fields>`;
    }
  });

export default updateList;
