/**
 * @name updateListItems
 * @description Adds, deletes, or updates the specified items in a list on the current site..
 * @param {object} options
 * @param {string} options.listName - The list display name.
 * @param {array} options.methods - An array of objects containing {Cmd: "New|Update|Delete", valuepairs and ID}.
 * @param {string} [options.webURL=""] - The web site url
 * @param {boolean} [options.async=true] - Synchronisation Type
 * @param {boolean} [options.parse=false] - XML will be parsed with items return
 * @param {number} [options.batchSize=400] - The batch size. A value of 0 or null will allow an infinite batch size.
 * @param {function} [options.getBatchData] - A callback function is called when batches are ready to be sent to the server
 * @returns {Promise} Returns a Promise XML result.
 * @example
 * updateListItems({
 *  listName: "Announcements",
 *  methods: [
 *    {
 *     cmd: "New",
 *     valuepairs: [
 *      // StaticName, Value
 *      ["Title", "My App"],
 *      ["Name", "Ross"]]
 *    },
 *    {
 *      cmd: "Update",
 *      ID: 5,
 *      valuepairs: [["Title", "My App"], ["Name", "Ross"]]
 *    },
 *    {
 *     cmd: "Delete",
 *     ID: 23,
 *    }
 *  ]
 * });
 */
const updateListItems = ({
  listName,
  webURL = "",
  async = true,
  parse = false,
  methods = [],
  batchSize = 400,
  getBatchData = null,
} = {}) => {
  return new Promise((resolve, reject) => {
    // Validate request
    if (methods.length === 0 || !methods[0].cmd) {
      reject("Invalid Methods. Unable to update items: ", methods);
    }

    // Create batches array literal
    // * Note batching is used when updating large lists to reduce risk of server timeouts
    let batches = [[]];

    // Create counters
    const itemsToUpdate = methods.length;
    let index = 1;
    let batchIndex = 0;
    let errors = 0;
    let success = 0;

    // Iterate through methods and push to batches
    methods.forEach((method) => {
      // Get Values
      const cmd = method.cmd;
      const ID = method.ID;
      const valuepairs = method.valuepairs;

      // Open method tag
      let str = `<Method ID='${index}' Cmd='${cmd}'>`;

      // If update or delete and ID is defined, add Field ID
      if (cmd !== "New" && ID) str += `<Field Name='ID'>${ID}</Field>`;

      // Add valuepairs. Delete requires the FileRef
      if (valuepairs.length > 0) {
        str += valuepairs
          .map(([name, value]) => `<Field Name='${name}'>${value}</Field>`)
          .join("");
      }

      // Close method tag
      str += `</Method>`;

      // Push string (method) to batch
      batches[batchIndex].push(str);

      // If batchSize is reached
      if (batchSize > 0 && index >= batchSize) {
        // Reset counters and create new batch
        index = 1;
        batchIndex += 1;
        batches.push([]);
      } else {
        // Increase index
        index += 1;
      }
    });

    // If callback is defined
    if (getBatchData) {
      getBatchData({
        batchesProcessed: 0,
        batchesTotal: batches.length,
        batchSize,
        itemsToUpdate,
        errors,
        success,
        results: {
          success: [],
          errors: [],
        },
      });
    }

    // Array literal to store all API responses
    let response = [];

    // Clone batches to iterate
    const array = [...batches];

    // Aync loop
    (function loop(i = 0) {
      // If array is empty
      if (!array || array.length === 0) {
        // Flatten array and resolve
        resolve([].concat.apply([], response));
        // End function
        return;
      }
      // Get first array item
      let batch = array.shift();

      // Send web request
      $().SPServices({
        operation: "UpdateListItems",
        webURL,
        async,
        listName,
        updates: `<Batch OnError='Continue'>${batch}</Batch>`,
        completefunc: function (xData, status) {
          // If server error
          if (status === "error") {
            errors += batch.length;

            console.error(
              "Error processing batch updates: ",
              xData.responseXML,
              "batch",
              batch
            );
          }

          // Array literal to store parsed results
          let results = {
            errors: [],
            success: [],
          };

          // If parse
          if (parse && status !== "parsererror" && xData.responseXML) {
            [...xData.responseXML.getElementsByTagName("Result")].forEach(
              (result) => {
                // Get the method ID
                const methodID = result.getAttribute("ID");
                // Get row
                let row = result.querySelector("row");

                // Create item object literal
                let item = {};

                // If row is truthy
                if (row) {
                  // Increase success count
                  success += 1;
                  // Create item props
                  item = [...row.attributes].reduce((object, node) => {
                    const name = node.name.split("ows_")[1];
                    if (name) {
                      object[name] = node.value;
                    }
                    return object;
                  }, {});
                  item.status = "success";
                }

                // Get Error Text (If found, an error occured)
                let errorText = [
                  ...result.getElementsByTagName("ErrorText"),
                ][0];

                // If an error is found, add error data to item object
                if (errorText) {
                  // Get Error Code
                  let code = [...result.getElementsByTagName("ErrorCode")][0];
                  // Increase erorr count
                  errors += 1;
                  // Create error props
                  item = {
                    ...item,
                    code: (code || {}).textContent,
                    message: (errorText || {}).textContent,
                    status: "error",
                    methodID,
                  };
                }

                if (item.status === "error") {
                  results.errors.push(item);
                } else {
                  results.success.push(item);
                }
              }
            );

            // If callback is defined
            if (getBatchData) {
              getBatchData({
                batchesProcessed: i + 1,
                batchesTotal: batches.length,
                batchSize,
                itemsToUpdate,
                success,
                errors,
                results,
              });
            }

            // Resolve item
            response.push(results);

            // Loop
            loop((i += 1));
          } else {
            // If callback is defined
            if (getBatchData) {
              getBatchData({
                batchesProcessed: i + 1,
                batchesTotal: batches.length,
                batchSize,
                itemsToUpdate,
                success,
                errors,
                results,
              });
            }
            // Resolve xml
            response.push(xData.responseXML);

            // Loop
            loop((i += 1));
          }
        },
      });
    })();
  });
};

export default updateListItems;
