// External
const { $ } = window;

/**
 * @name resolvePrincipals
 * @description esolves an e-mail address or other form of user identification (ID) into a PrincipalInfo object.
 * @param {Object} object
 * @param {String|Array<String>} principalKeys A string or an array of logon names of the principal
 * @param {String} [principalType="User"] The Principal type must be "User" or "SharePointGroup".
 * @param {String} [webURL=window.$().SPServices.defaults.webURL] The relative webURL
 * @param {Boolean} [parse=true] If false, the response will be returned in raw XML format
 * @param {Boolean} [async=true] If false, the request will be synchronous.
 * @param {Boolean} [addToUserInfoList=false] If true, the user will be added to the User Information List.
 * @returns {Object|Array<Object>} If parsing is true an array of objects is returns. If false an XML object is returned.
 * @returns
 */
const resolvePrincipals = ({
  webURL = $().SPServices.defaults.webURL || "",
  principalType = "User",
  principalKeys,
  async = true,
  parse = true,
  addToUserInfoList = false,
} = {}) =>
  new Promise((resolve, reject) => {
    // If principalKeys is a string, cast to an array
    if (typeof principalKeys === "string") principalKeys = [principalKeys];

    // If principalType is not User or SharePointGroup
    if (!["User", "SharePointGroup"].includes(principalType)) {
      // Reject with error
      return reject(
        new Error(`The principalType of ${principalType} was incorrect.`)
      );
    }

    // If principalKeys is not an array
    if (!Array.isArray(principalKeys)) {
      // Reject with error
      return reject(new Error("No searchText was provided"));
    }

    // Convert principalKeys to an string
    principalKeys = principalKeys.map((str) => `<string>${str}</string>`);

    // Send request
    $().SPServices({
      operation: "ResolvePrincipals",
      webURL,
      principalKeys,
      principalType,
      async,
      addToUserInfoList,
      completefunc: function (xData, status) {
        // If an error occurred
        if (status === "error") {
          // Reject with error
          return reject({
            error: new Error("Error occurred when searching principals"),
            xData,
          });
        }

        try {
          // Get response
          let response = xData.responseXML;
          // If parse is truthy
          if (parse) {
            // Parse response
            response = [
              ...response.querySelectorAll(
                "ResolvePrincipalsResult > PrincipalInfo"
              ),
            ].map((principal) =>
              // Create object from children (childNodes works with IE)
              [...principal.childNodes].reduce((object, node) => {
                // Get object key
                const key = node.nodeName;

                // Create string literal to store value
                let value = "";

                // Switch by key (nodeName)
                switch (key) {
                  case "MoreMatches":
                    // Set value to be an array of objects containing user principals
                    value = [...node.querySelectorAll("PrincipalInfo")].map(
                      (match) =>
                        [...match.childNodes].reduce((object, node) => {
                          object[node.nodeName] = node.textContent || "";

                          if (node.nodeName === "IsResolved") {
                            object[node.nodeName] = node.textContent === "true";
                          } else {
                            object[node.nodeName] = node.textContent || "";
                          }

                          return object;
                        }, {})
                    );
                    break;
                  case "IsResolved":
                    value = node.textContent === "true";
                    break;

                  default:
                    // Set value as node text content
                    value = node.textContent || "";
                    break;
                }

                // Assign value to object
                object[key] = value;

                // Return object/accumulator
                return object;
              }, {})
            );
          }

          // Resolve response
          resolve(response);
        } catch (error) {
          // Reject with error
          reject({ error, xData });
        }
      },
    });
  });

export default resolvePrincipals;
