// External
const { $ } = window;

/**
 * @name searchPrincipals
 * @description Returns an array of PrincipalInfo objects from the SPUserCollection for the Web site. All instances have the principal logon name specified in searchText.
 * @param {Object} object
 * @param {String} searchText The Principal logon name.
 * @param {Number} [maxResults=10] The maximum results to be requested
 * @param {String} [webURL=window.$().SPServices.defaults.webURL] The relative webURL
 * @param {Boolean} [parse=true] If false, the response will be returned in raw XML format
 * @param {Boolean} [async=true] If false, the request will be synchronous.
 * @returns {Object|Array<Object>} If parsing is true an array of objects is returns. If false an XML object is returned.
 * @returns
 */
const searchPrincipals = ({
  searchText,
  maxResults = 100,
  webURL = $().SPServices.defaults.webURL || "",
  parse = true,
  async = true,
} = {}) =>
  new Promise((resolve, reject) => {
    // If no searchText is provided
    if (!searchText) {
      // Reject with error
      return reject(new Error("No searchText was provided"));
    }

    // Send request
    $().SPServices({
      operation: "SearchPrincipals",
      webURL,
      searchText,
      maxResults,
      async,
      SPPrincipalType: "SPPrincipalType.User",
      completefunc: function (xData, status) {
        // If an error occurred
        if (status === "error") {
          // Reject with error
          return reject({
            error: new Error("Error occurred when searching principals"),
            xData,
          });
        }

        try {
          // Get response
          let response = xData.responseXML;
          // If parse is truthy
          if (parse) {
            // Parse response
            response = [...response.querySelectorAll("PrincipalInfo")].map(
              (principal) =>
                // Create object from children (childNodes works with IE)
                [...principal.childNodes].reduce((object, node) => {
                  if (node.nodeName === "IsResolved") {
                    object[node.nodeName] = node.textContent === "true";
                  } else {
                    object[node.nodeName] = node.textContent || "";
                  }
                  return object;
                }, {})
            );
          }

          // Resolve response
          resolve(response);
        } catch (error) {
          // Reject with error
          reject({ error, xData });
        }
      },
    });
  });

export default searchPrincipals;
