/**
 * @name getVersions
 * @description Returns information about the versions of the specified file.
 * @param {Object} payload The configuration payload object
 * @param {String} payload.fileName A string that contains the site-relative URL of the file in the form library/folder/File_Name (Do Not include the webURL).
 * @param {String} [payload.webURL=window.$().SPServices.defaults.webURL] The web site url
 * @param {Boolean} [payload.async=true] If false, the request will be synchronous (deprecated)
 * @param {Boolean} [payload.parse=false] If true, the response will be passed
 * @returns {Promise<Array|Object>} Returns an XML document unless parsing is true, then an array of objects is returned
 */
const getVersions = ({
  fileName,
  webURL = window.$().SPServices.defaults.webURL || "",
  async = true,
  parse = false,
} = {}) =>
  new Promise((resolve, reject) => {
    // Validate request
    if (!fileName) return reject("Missing fileName. Unable to send request");

    // Send Request
    $().SPServices({
      operation: "GetVersions",
      webURL,
      async,
      fileName,
      completefunc: function (xData, status) {
        // Create reponse variable
        let response = xData.responseXML || null;

        switch (status) {
          case "error":
            if (response) {
              try {
                response = response.querySelector("errorstring").textContent;
              } catch (error) {
                response = `Unable to get response data from the server: ${error.message}`;
              }
            }
            return reject(response);

          case "success":
            // If parse is true
            if (parse) {
              // Parse the response
              response = [...response.querySelectorAll("result")].map(
                (result) => {
                  return [...result.attributes].reduce(
                    (object, { nodeName, nodeValue }) => {
                      object[nodeName] = nodeValue;
                      return object;
                    },
                    {}
                  );
                }
              );
            }
            // Resolve the response
            resolve(response);
            break;

          default:
            resolve(xData.responseXML);
            break;
        }
      },
    });
  });

export default getVersions;
