// Utils
import getNodeAttributes from "../../utils/general/getNodeAttributes";

/**
 * @name getViewHtml
 * @description Get HTML from a view
 * @param {Object} payload
 * @param {String} [payload.webURL=window.$().SPServices.defaults.webURL] The web site url.
 * @param {Boolean} [payload.async=true] If false, the request will be synchronous.
 * @param {Boolean} [payload.parse=true] If false, the raw xml document will be resolved.
 * @returns {Promise<Object>} Resolves an object.
 * @example
 * getViewHtml({
 *  listName: "Announcements",
 *  viewName: "{315748D1-8FF2-4252-B88C-995B9179FB72}"
 * }).then(res => console.info(res))
 */
const getViewHtml = ({
  webURL = window.$().SPServices.defaults.webURL,
  async = true,
  parse = true,
  listName,
  viewName,
} = {}) =>
  new Promise((resolve, reject) => {
    $().SPServices({
      operation: "GetViewHtml",
      webURL,
      listName,
      viewName,
      async,
      completefunc: function (xData, status) {
        try {
          // If error
          if (status !== "success") return reject(xData);

          // Get XML
          const xml = xData.responseXML;

          // If parse is false
          if (!parse) {
            // Resolve XML
            return resolve(xml);
          }

          // Get view Props
          const View = getNodeAttributes(xml.querySelector("View"));

          // Convert ViewFields into array
          const ViewFields = [...xml.querySelectorAll("ViewFields FieldRef")].map((node) =>
            node.getAttribute("Name")
          );

          // Convert Query into String
          const Query = new XMLSerializer().serializeToString(xml.querySelector("Query"));

          // Get Row Limit Element
          let RowLimit = xml.querySelector("RowLimit");

          // Check if paged
          const Paged = RowLimit ? RowLimit.getAttribute("Paged") : false;

          // Get Row Limit
          RowLimit = (RowLimit || {}).textContent || "";

          // Get Toolbar
          const Toolbar = xml.querySelector("Toolbar").getAttribute("Type");

          // Resolve
          resolve({ ...View, ViewFields, Query, RowLimit, Paged, Toolbar });
        } catch (error) {
          reject(xData, error);
        }
      },
    });
  });

export default getViewHtml;
