import getWebPartPagesError from "../../utils/errors/getWebPartPagesError";
import getWebPartPagesOptions from "../../utils/options/getWebPartPagesOptions";

/**
 * @name getWebPartProperties
 * @description Returns an XML string of all Web Parts associated with a Web Part Page.
 * @required pageUrl
 * @param {object} options
 * @param {string} options.pageUrl - A string containing the URL of the Web Part Page.
 * - Required: Yes
 * @param {boolean} [options.parse=false] - Parse the response and return as JSON.
 * - Required: No
 * - Default: false
 * @param {string} [options.webURL=""] - The web site url.
 * - Required: No
 * - Default: "" (current site)
 * @param {boolean} [options.async=true] - Synchronisation Type.
 * - Required: No
 * - Default: true (asynchronous)
 * @returns {Promise} Returns a Promise XML result.
 */
const getWebPartProperties = options => {
  return new Promise((resolve, reject) => {
    const {
      webURL,
      pageUrl,
      parse,
      timeout,
      storage,
      async
    } = getWebPartPagesOptions(options);
    $().SPServices({
      operation: "GetWebPartProperties",
      webURL,
      pageUrl,
      storage,
      parse,
      timeout,
      async,
      completefunc: function(xData, status) {
        if (status === "success") {
          // If parsing is not required.
          if (!parse) {
            resolve(xData.responseXML);
          } else {
            // If parsing is required
            let webParts = [];

            $(xData.responseXML)
              .SPFilterNode("WebPart")
              .each(function() {
                // Current Row
                const row = this;

                // Web Part
                let webPart = {};

                // Get Attributes
                const attributes = row.attributes;
                for (let i = 0; i < attributes.length; i++) {
                  const attribute = attributes[i];
                  const key = `_${attribute.name}`;
                  webPart[key] = attribute.value;
                }

                // Get XML Children Nodes
                $(row)
                  .children()
                  .each(function() {
                    webPart[this.nodeName] = $(this).text();
                  });
                webParts.push(webPart);
              });
            resolve(webParts);
          }
        } else {
          reject(getWebPartPagesError(xData));
        }
      }
    });
  });
};

export default getWebPartProperties;
