// Utils
import getNodeAttributes from "../../utils/general/getNodeAttributes";
import convertStringToXml from "../../utils/general/convertStringToXml";

/**
 * @name getTemplatesForItem
 * @param {Object} payload
 * @param {String} payload.itemURL The URL location of an item on which a workflow is being run.
 * @param {String} [payload.webURL=window.$().SPServices.defaults.webURL] The web site url.
 * @param {Boolean} [payload.async=true] If false, the request will be synchronous.
 * @param {Boolean} [payload.parse=true] If false, the raw xml document will be resolved.
 * @returns {Promise<Object>} Resolves a promised object contains workflow data or a XML document.
 * getTemplatesForItem({
 *  webURL: "/sites/development",
 *  itemURL: "http://kingsolutions/sites/development/Lists/DemoList/1_.000",
 *  parse: true,
 *  async: true,
 * })
 */
const getTemplatesForItem = ({
  async = true,
  itemURL,
  webURL = window.$().SPServices.defaults.webURL,
  parse = true,
}) =>
  new Promise((resolve, reject) => {
    $().SPServices({
      operation: "GetTemplatesForItem",
      webURL,
      item: itemURL,
      async,
      completefunc: function (xData, status) {
        // If status is not success
        if (status === "error") {
          // Return with rejected data
          return reject(xData, status);
        }

        // Get XML document
        let xml = xData.responseXML;

        // If XML is not defined
        if (!xml) {
          // Convert String to XML
          xml = convertStringToXml(xData.responseText || "");

          // If XML is not defined, reject with error
          if (!xml) {
            // Return/reject error
            return reject(xData, status);
          }
        }

        // If parsing is not required
        if (!parse) {
          // Resolve raw xml document
          return resolve(xml);
        }

        // Create response object
        let response = {};

        // Get web
        response.Web = getNodeAttributes(xml.querySelector("Web"));
        // Get list
        response.List = getNodeAttributes(xml.querySelector("List"));

        // Get Workflow Templates
        response.WorkflowTemplates = [
          ...xml.querySelectorAll("WorkflowTemplate"),
        ].map((node) => {
          // Get current template
          let object = getNodeAttributes(node);
          // Get Template ID Set
          object.WorkflowTemplateIdSet = getNodeAttributes(
            node.querySelector("WorkflowTemplateIdSet")
          );
          return object;
        });

        // Return response object
        return resolve(response);
      },
    });
  });

export default getTemplatesForItem;
