// Utils
import getNodeAttributes from "../../utils/general/getNodeAttributes";
import convertStringToXml from "../../utils/general/convertStringToXml";

/**
 * @name getWorkflowDataForItem
 * @description Gets the workflow data for a specific user for a given SharePoint item.
 * @param {Object} payload
 * @param {String} payload.itemURL The URL location of an item on which a workflow is being run.
 * @param {String} [payload.webURL=window.$().SPServices.defaults.webURL] The web site url.
 * @param {Boolean} [payload.async=true] If false, the request will be synchronous.
 * @param {Boolean} [payload.parse=true] If false, the raw xml document will be resolved.
 * @returns {Promise<Object>} Resolves a promised object contains workflow data or a XML document.
 * @example
 * getWorkflowDataForItem({
 *  webURL: "/sites/development",
 *  itemURL: "http://kingsolutions/sites/development/Lists/DemoList/1_.000",
 *  parse: true,
 *  async: true,
 * })
 */
const getWorkflowDataForItem = ({
  async = true,
  itemURL,
  parse = true,
  webURL = window.$().SPServices.defaults.webURL,
}) =>
  new Promise((resolve, reject) => {
    $().SPServices({
      operation: "GetWorkflowDataForItem",
      webURL,
      item: itemURL,
      async,
      completefunc: function (xData, status) {
        // If error
        if (status === "error") {
          // Return/reject error
          return reject(xData, status);
        }
        // Get xml from response
        let xml = xData.responseXML;

        // If XML is not defined
        if (!xml) {
          // Convert String to XML
          xml = convertStringToXml(xData.responseText || "");

          // If XML is not defined, reject with error
          if (!xml) {
            // Return/reject error
            return reject(xData, status);
          }
        }

        // If parsing is not required
        if (!parse) {
          // Return raw xml response
          return resolve(xml);
        }

        // Create response object
        let response = {};

        // Get web
        response.Web = getNodeAttributes(xml.querySelector("Web"));

        // Get list
        response.List = getNodeAttributes(xml.querySelector("List"));

        // Get active Workflows Data
        response.ActiveWorkflowsData = [
          ...xml
            .querySelector("ActiveWorkflowsData")
            .querySelectorAll("Workflow"),
        ].map((workflow) => {
          return [...workflow.attributes].reduce((object, node) => {
            // Create key string
            let key = node.name;

            // Get value
            let value = node.value || "";

            // Create regular expression for status key
            const statusRegExp = new RegExp(/Status[0-9]/);

            // If status key, remove any numbers at the end
            if (statusRegExp.test(key)) key = "Status";

            // Switch by key
            switch (key) {
              // Decode the URL string
              case "StatusPageUrl":
                value = window.decodeURIComponent(value);
                break;
              // Create an array of truthy values
              case "Status":
                // If status is not defined, create status
                if (!object.Status) object.Status = [];
                // If value is truthy
                if (value) {
                  // Push value to status
                  object.Status.push(value);
                }
                // Ensure statis remains as an array
                value = object.Status;
                break;

              default:
                break;
            }

            // Update the object
            object[key] = value;

            // Return the object
            return object;
          }, {});
        });

        // Get Workflow Templates
        response.WorkflowTemplates = [
          ...xml.querySelectorAll("WorkflowTemplate"),
        ].map((node) => {
          // Get current template
          let object = getNodeAttributes(node);
          // Get Template ID Set
          object.WorkflowTemplateIdSet = getNodeAttributes(
            node.querySelector("WorkflowTemplateIdSet")
          );
          return object;
        });

        // Return response object
        return resolve(response);
      },
    });
  });

export default getWorkflowDataForItem;
