/**
 * @name startWorkflow
 * @description Gets the workflow data for a specific user for a given SharePoint item.
 * @param {Object} payload
 * @param {String} payload.itemURL The URL location of an item on which a workflow is being run.
 * @param {String} payload.templateId The globally unique identifier (GUID) of a template.
 * @param {String} [payload.workflowParameters="<root/>"] The workflow variables to be set.
 * @param {String} [payload.webURL=window.$().SPServices.defaults.webURL] The web site url.
 * @param {Boolean} [payload.async=true] If false, the request will be synchronous.
 * @param {Boolean} [payload.parse=true] If false, the raw xml document will be resolved.
 * @returns {Promise<Object|Boolean>} Resolves a promise constain true if started or xData is rejected. If parsing is not required a XML document is resovled.
 * @example
 * // Start a workflow without setting variables
 * startWorkflow({
 *  webURL: "/sites/development",
 *  itemURL: "http://kingsolutions/sites/development/Lists/DemoList/1_.000",
 *  templateID: "{b180099a-3dde-4ae8-988e-1914da0565ab}",
 *  workflowParameters: "<root />",
 *  parse: true,
 *  async: true,
 * })
 *
 * // Start a workflow with setting variables of "subject" and "body"
 * startWorkflow({
 *  webURL: "/sites/development",
 *  itemURL: "http://kingsolutions/sites/development/Lists/DemoList/1_.000",
 *  templateID: "{b180099a-3dde-4ae8-988e-1914da0565ab}",
 *  workflowParameters: `<Data>
 *      <subject>Workflow Parameter Test</subject>
 *      <body>This is the body text of a worflow email notification</body>
 *    </Data>`,
 *  parse: true,
 *  async: true,
 * })
 */
const startWorkflow = ({
  async = true,
  itemURL,
  parse = true,
  templateId,
  webURL = window.$().SPServices.defaults.webURL,
  workflowParameters = "<root />",
}) =>
  new Promise((resolve, reject) => {
    $().SPServices({
      webURL,
      operation: "StartWorkflow",
      async,
      item: itemURL,
      templateId,
      workflowParameters,
      completefunc: function (xData, status) {
        // If status is not success
        if (status !== "success") {
          // Return with rejected data
          return reject(xData, status);
        }

        // Get XML document
        const xml = xData.responseXML;

        // If parsing is not required
        if (!parse) {
          // Resolve raw xml document
          return resolve(xml);
        }

        // Resolve true if started
        resolve(true);
      },
    });
  });

export default startWorkflow;
