// Libraries
import CamlBuilder from "spcamljs";

// API
import getList from "../../core/lists/getList";
import getListItems from "../../core/lists/getListItems";
import getMultiListItems from "./getMultiListItems";
import getThrottledListItems from "./getThrottledListItems";
import getThrottledMultiListItems from "./getThrottledMultiListItems";

// Utils
import _parseEncodedAbsUrl from "../general/parseEncodedAbsUrl";

/**
 * @name getItems
 * @description Get list items with advanced options for multi lists and thottle handling
 * @param {Object} payload The configuration payload object
 * @param {Array} payload.listNames An array of list display names as strings
 * @param {Boolean} [payload.parseFileDirRef] If true, the FileDirRef will be parsed into item properties. This will also include the FileDirRef viewfield.
 * @param {Boolean} [payload.debug=false] If true, batches are logged to the console.
 * @param {Boolean} [payload.throttledColumns=false] If true, queries will handle lookup columns over the threshold
 * @param {Boolean} [payload.throttledItems=true] If false, queries will not handle lists over the list view threshold
 * @param {Function} [payload.parseFunction] A callback function that allows item properties to be created/overriden whilst rows are parsed.
 * @param {boolean} [payload.async=true] - Synchronisation Type
 * @param {Number} [payload.maxBatchSize=null] - The maximum batch size. This will override the automatic batch sizing set by listViewThreshold.
 * @param {Number} [payload.CAMLRowLimit = 0] The maximum number of rows to retreive
 * @param {Number} [payload.timeout = 0] If greater than 0, a timeout (delay) in milliseconds will be used before each web request
 * @param {String} [payload.CAMLQuery="<Query />"] The CAML query as a XML string.
 * @param {String} [payload.CAMLQueryOptions="<QueryOptions></QueryOptions>"] The CAML query options as a XML string.
 * @param {String} [payload.webURL=window.$().SPServices.defaults.webURL] The web site url
 * @param {boolean} [payload.allowLongFieldNames=false] If true, the field names contained with values will not be trimmed to 32 characters.
 * This allows field names that exceed 32 characters to be used.
 * This is a rare use case where 2 or more fields share the exact same name for the first 32 characters. For example
 * 1. ThisIsAExtremelyLongTextFieldNam
 * 2. ThisIsAExtremelyLongTextFieldNam0
 */
const getItems = ({
  CAMLQuery = `<Query />`,
  CAMLQueryOptions = "<QueryOptions></QueryOptions>",
  CAMLRowLimit = 0,
  listNames,
  throttledColumns = false,
  throttledItems = true,
  viewFields = [],
  parseFunction,
  parseFileDirRef = false,
  maxBatchSize = null,
  async = true,
  webURL = window.$().SPServices.defaults.webURL,
  timeout = 0,
  debug = false,
  allowLongFieldNames = false,
} = {}) => {
  // If listNames are not defined, not an array or an empty array.
  if (!listNames || !Array.isArray(listNames) || listNames.length === 0) {
    // Log error and return empty string
    console.error(
      "You have not provided an array of listNames. Unable to proceed with query. An empty array will be returned"
    );
    return [];
  }

  // Create an index of list IDs and their corresponding index in the items array
  let indexes = {};

  // An array of items that are added to each batch
  let items = [];

  return new Promise((resolve) => {
    // If parseFileDirRef is required and viewfields does not include the field.
    if (parseFileDirRef) {
      if (!viewFields.includes("FileDirRef")) {
        // Push FileDirRef (Deprecated)
        viewFields.push("FileDirRef");
      }

      if (!viewFields.includes("EncodedAbsUrl")) {
        // Push Encoded URL. This replaces FileDirRef
        viewFields.push("EncodedAbsUrl");
      }
    }

    // Get fields from list
    getFields({
      webURL,
      listNames,
      throttledColumns,
      async,
    }).then((fields) => {
      // Create an array of view field batches
      const viewFieldBatches = fields
        ? // If the fields object is defined, validate the amount of throttled fields
          createViewFieldBatches(fields, viewFields)
        : // If throttledColumns are not required, add all viewFields to a single batch.
          [viewFields];

      // Create request object
      let request = {
        webURL,
        CAMLQuery,
        CAMLQueryOptions,
        CAMLRowLimit,
        maxBatchSize,
        debug,
        async,
      };

      // Create batch counters
      let batchCount = viewFieldBatches.length;
      let viewFieldBatchesIndex = 0;

      // Loop through all batches
      (function loop() {
        // If all batches have been processed
        if (!viewFieldBatches || viewFieldBatches.length === 0) {
          // Resolve items
          return resolve(items);
        }

        // Create viewFields array from batch
        const viewFields = viewFieldBatches.shift();

        // Update view fields for current batch
        request.CAMLViewFields = new CamlBuilder()
          .ViewFields(
            viewFields.map((field) => {
              const staticName = allowLongFieldNames
                ? field
                : field.slice(0, 32);
              return staticName;
            })
          )
          .ToString();

        // Send SharePoint Request
        sendRequest(request, listNames).then((rows) => {
          // Create a timeout/delay if a timeout isn't defined.
          // For large requests, default the timeout to 100ms
          let delay =
            timeout || (viewFields.length >= 40 && rows.length >= 1000)
              ? 100
              : 0;
          setTimeout(() => {
            // Increase index
            viewFieldBatchesIndex += 1;

            // Parse rows
            parseRows(
              rows,
              viewFields,
              viewFieldBatchesIndex === batchCount,
              batchCount
            );

            // Continue loop
            loop();
          }, delay);
        });
      })();

      /**
       * @name parseRows
       * @description Converts rows into items
       * @param {Array} rows An array of rows as xml
       * @param {Array<String>} fields An array of field names as strings
       * @returns {Array<Object>} Returns an array of objects
       */
      function parseRows(rows, fields, isLastBatch) {
        // Iterate through each row
        rows.forEach((row, index) => {
          // Create item object from viewFields
          let item = fields.reduce((object, field) => {
            const staticName = allowLongFieldNames ? field : field.slice(0, 32);
            object[field] = row.getAttribute(`ows_${staticName}`) || "";
            return object;
          }, {});

          // Assign item ID (used for matching)
          item.ID = row.getAttribute("ows_ID");

          // Add to indexes
          indexes[item.ID] = indexes[item.ID] || index;

          // Find matched item
          const matchedItem = items[indexes[item.ID]];

          // If matched item
          if (matchedItem && !matchedItem.EncodedAbsUrl) {
            // Get EncodedAbsUrl
            const EncodedAbsUrl = row.getAttribute("ows_EncodedAbsUrl") || "";
            // If EncodedAbsUrl is defined
            if (EncodedAbsUrl) {
              // Assign EncodedAbsUrl to matched item
              matchedItem.EncodedAbsUrl = EncodedAbsUrl;
            }
          }

          // If parseFileDirRef is true
          if (isLastBatch && parseFileDirRef) {
            item = {
              ...(items[indexes[item.ID]] || {}),
              ...item,
              ..._parseEncodedAbsUrl(item.EncodedAbsUrl),
            };
          }

          // If a parseFunction callback is defined
          if (parseFunction) item = parseFunction(item);

          // Add to array
          items[index] = {
            ...(items[indexes[item.ID]] || {}),
            ...item,
          };
        });
      }
    });
  });

  /**
   * @name sendRequest
   * @description Sends the get items request(s) to SharePoint
   * @param {Object} request The request props.
   * @param {Array} listNames An array of list display names
   * @returns {Promise<Array>} Returns an array of items
   */
  function sendRequest(request, listNames) {
    return new Promise((resolve, reject) => {
      if (!throttledItems) {
        if (listNames.length > 1) {
          getMultiListItems({
            listNames,
            timeout,
            ...request,
          }).then(resolve);
        } else {
          getListItems({
            listName: listNames[0],
            ...request,
          })
            .then((responseXML) =>
              resolve([...responseXML.querySelectorAll("row")])
            )
            .catch((error) => {
              console.error(
                "Unable to get list items, an empty array has been returned :",
                error
              );
              resolve([]);
            });
        }
      } else {
        // Send correct request
        if (listNames.length > 1) {
          getThrottledMultiListItems({
            listNames,
            timeout,
            ...request,
          }).then(resolve);
        } else {
          // Send web request
          getThrottledListItems({
            listName: listNames[0],
            timeout,
            ...request,
          })
            .then(resolve)
            .catch(reject);
        }
      }
    });
  }
};

/**
 * @name getFields
 * @param {Object} payload
 * @param {Array} payload.listNames An array of list display names
 * @param {String} payload.webURL The web URL
 * @param {Boolean} payload.throttledColumns If false, list fields will be resolved as null
 * @param {boolean} [payload.async=true] - Synchronisation Type
 * @returns {Object|Null} Returns an object of field types or null if columns are not throttled
 */
function getFields({ listNames, webURL, throttledColumns, async = true }) {
  // Create promise
  return new Promise((resolve) => {
    // If columns aren't throttled, resolve fields as null
    if (!throttledColumns) return resolve(null);

    // Clone the listNames
    let lists = [...listNames];

    // Create an object literal to store future field data types
    let fields = {};

    // Iterate through all listNames
    (function loop() {
      // If the lists are null or an empty array
      if (!lists || lists.length === 0) {
        // Resolve the fields object
        return resolve(fields);
      }

      // Get the listName from the lists array
      const listName = lists.shift();

      // Get list data from SharePoint
      getList({
        webURL,
        listName,
        async,
      }).then((res) => {
        // Parse list fields and add to the fields object
        [...res.querySelectorAll("Field")].forEach(
          (field) =>
            (fields[field.getAttribute("StaticName")] =
              field.getAttribute("Type") || "")
        );

        // Loop to next list
        loop();
      });
    })();
  });
}

/**
 * @name createViewFieldBatches
 * @description Creates batches of viewFields to ensure thresholds are not exceeded
 * @param {Object} fields An object of field types
 * @param {Array} viewFields An array of list display names as strings
 * @returns {Array} Returns a multi-dimensional array of viewfield batches.
 */
function createViewFieldBatches(fields, viewFields) {
  // Array to store fields
  let array = [[]];

  // Throttled field counter
  let throttledCount = 0;

  // Iterate through all view fields to build array
  viewFields.forEach((field) => {
    // Check if field is throttled
    let isThrottled =
      fields[field] &&
      ["User", "UserMulti", "Calculated", "Lookup", "WorkflowStatus"].includes(
        fields[field]
      );

    // If not throttled
    if (!isThrottled) {
      // Push field to first batch
      array[0].push(field);
    } else {
      // Increase throttled counter
      throttledCount += 1;
      // If less than 8 throttled fields have been included in viewFields
      if (throttledCount <= 8) {
        // Push field to first batch
        array[0].push(field);
      } else {
        // If the throttled array exceeds 7, push a new arrya
        if (array[array.length - 1].length >= 8) {
          array.push([]);
        }
        // Push to throttled array
        array[array.length - 1].push(field);
      }
    }
  });

  // Return the array of view fields
  return array;
}

export default getItems;
