import getListItems from "../../core/lists/getListItems";

/**
 * @name getMultiListItems
 * @description Returns information about items in multiple list based on the same specified query per list.
 * @required listNames
 * @param {Object} options
 * @param {Array} options.listNames - An array of list display names.
 * @param {String} [options.webURL=""] - The web site url
 * @param {String} [options.CAMLViewFields="<ViewFields Properties='True' />"] - The view fields as an XML string
 * @param {String} [options.CAMLQuery="<Query></Query>"] - The CAML query as an XML string
 * @param {String} [options.CAMLQueryOptions="<QueryOptions></QueryOptions>"] - Additional query options as an XML string.
 * @param {Boolean} [payload.debug=false] If true, batches are logged to the console.
 * @param {Number} [options.CAMLRowLimit=0] - The number of rows to retrieve
 * @param {Number} [options.timeout=10] - The timeout (pause) between iterations when parsing items.
 * @param {Boolean} [options.async=true] - Synchronisation Type
 * @param {Boolean} [options.parseItems=false] - XML will be parsed with items return
 * @param {Boolean} [options.rawXml=false] - XML will not be filtered to rows and returned with full response.
 * @returns {Promise} Returns a Promise XML result.
 * @example - // Query list on same site
 * getListItems({
 *    listNames: ["Announcements", "Tasks"],
 * });
 * @example - // Query list on another site
 * getListItems({
 *  listNames: ["Announcements", "Tasks"],
 *  webURL: "http://objectpoint.com/sites/defence"
 * });
 * @example - // Advanced query
 * getListItems({
 *  listNames: ["Announcements"],
 *  webURL: "http://objectpoint.com/sites/defence",
 *  CAMLQuery: '<Query><Where><Eq><FieldRef Name="FirstName" /><Value Type="Text">Mapero</Value></Eq></Where></Query>',
 *  CAMLViewFields: '<ViewFields><FieldRef Name="Title" /></ViewFields>'
 *  parseItems: true
 * });
 */
const getMultiListItems = (options) => {
  return new Promise((resolve) => {
    // Ensure options.listNames is an array
    if (options.listNames && !Array.isArray(options.listNames))
      options.listNames = [options.listNames];

    // Store XML rows
    const rows = [];

    // Create Lists Array (used to iterate through each list)
    let lists = [...options.listNames];

    // Remove options.listNames
    delete options.listNames;

    // Begin Iterating Through Each List
    (function loop() {
      // If all lists have been requested. End loop and resolve.
      if (!lists || lists.length === 0) {
        // Convert Multi Dimensional Array to 1 Dimension Array and Resolve;
        resolve([].concat(...rows));
        return;
      }

      // Get List Names from List Names Array
      options.listName = lists.shift();

      // Get List Items for List
      getListItems(options)
        .then((xml) => {
          // Push XML response to rows constant
          if (options.parseItems) {
            rows.push(xml);
          } else {
            // If raw xml is request. Do not split filter rows.
            if (options.rawXml) {
              rows.push(xml);
            } else {
              // Else, push filtered rows
              rows.push($(xml).SPFilterNode("z:row").toArray());
            }
          }

          // Loop to next list
          loop();
        })
        .catch((error) => {
          console.error(
            "Unable to get list items, an empty array has been returned:",
            error
          );
          loop();
        });
    })();
  });
};

export default getMultiListItems;
