import getLastItemID from "./getLastItemID";
import getListItems from "../../core/lists/getListItems";
import parseListItems from "./parseListItems";
import getListsOptions from "../options/getListsOptions";
import LIST_VIEW_THRESHOLD from "../../constants/lists/listViewThreshold";
import CamlBuilder from "spcamljs";

const buildQuery = (fromID, toID, query) => {
  if (query.length > 20) {
    return CamlBuilder.FromXml(query)
      .ModifyWhere()
      .AppendAnd()
      .CounterField("ID")
      .GreaterThan(fromID)
      .And()
      .CounterField("ID")
      .LessThanOrEqualTo(toID)
      .ToString();
  } else {
    return `<Query>
    ${new CamlBuilder()
      .Where()
      .CounterField("ID")
      .GreaterThan(fromID)
      .And()
      .CounterField("ID")
      .LessThanOrEqualTo(toID)
      .ToString()}
    </Query>`;
  }
};

/**
 * @name batchGetListItems
 * @param {Boolean} [payload.debug=false] If true, batches are logged to the console.
 * @param {*} payload.batches
 * @param {*} payload.listViewThreshold
 * @param {*} payload.listName
 * @param {*} payload.webURL
 * @param {*} payload.query
 * @param {*} payload.CAMLViewFields
 * @param {*} payload.CAMLQueryOptions
 * @param {*} payload.rawXml
 * @param {*} payload.timeout
 */
const batchGetListItems = ({
  batches,
  listViewThreshold,
  listName,
  webURL,
  query,
  CAMLViewFields,
  CAMLQueryOptions,
  rawXml,
  timeout = 0,
  debug = false,
}) => {
  return new Promise((resolve, reject) => {
    // ID counters
    let fromID = 0;
    let toID = 0 + listViewThreshold;

    // Response array
    let response = [];

    // Batch count
    let batchCount = 0;

    // Self invoking loop
    (function loop() {
      // Create CAMLQuery
      const CAMLQuery = buildQuery(fromID, toID, query);

      const payload = {
        listName,
        webURL,
        CAMLQuery,
        CAMLViewFields,
        CAMLRowLimit: listViewThreshold,
        CAMLQueryOptions,
        debug,
        async: false,
      };

      // If debug is true
      if (debug) {
        console.info(`Get Throttled List Items Batch: ${batchCount} :>> `, {
          fromID,
          toID,
          ...payload,
        });
      }

      // Increase counter and IDs
      fromID += listViewThreshold;
      toID += listViewThreshold;
      batchCount += 1;

      // Request items
      getListItems(payload)
        .then((xml) => {
          if (rawXml) {
            response.push(xml);
          } else {
            response.push($(xml).SPFilterNode("z:row").toArray());
          }
          // If batchCount is less then total
          if (batchCount < batches) {
            // If timeout is defined
            if (timeout) {
              // Set timeout
              setTimeout(() => {
                // Call loop after timeout
                loop();
              }, timeout);
            } else {
              // No timeout is specified, call timeout immediately
              loop();
            }
          } else {
            resolve([].concat.apply([], response));
          }
        })
        .catch((err) => {
          reject(err);
        });
    })();
  });
};

/**
 * @name getListItems
 * @description Returns information about items in the list based on the specified query.
 * @required listName
 * @param {Object} options
 * @param {String} options.listName - The list display name.
 * @param {String} [options.webURL=""] - The web site url
 * @param {Boolean} [options.async=true] - Synchronisation Type
 * @param {String} [options.CAMLViewFields="<ViewFields Properties='True' />"] - The view fields as an XML string
 * @param {String} [options.CAMLQuery="<Query></Query>"] - The CAML query as an XML string
 * @param {String} [options.CAMLQueryOptions="<QueryOptions></QueryOptions>"] - Additional query options as an XML string.
 * @param {Boolean} [options.parseItems=false] - XML will be parsed with items return
 * @param {Number} [options.maxBatchSize=null] - The maximum batch size. This will override the automatic batch sizing set by listViewThreshold.
 * @param {Boolean} [options.rawXml=false] - Returns an array of raw responseXML data.
 * @param {Number} [options.timeout=0] - The timeout between batch requests
 * @param {Boolean} [options.viewColumns=null] - Only columns that are selectable in views will be returned (only if parsing is true)
 * @returns {Promise} Returns a Promise which resolves or array of xml or items.
 */
const getThrottledListItems = (options) => {
  return new Promise((resolve, reject) => {
    const {
      listName,
      webURL,
      async,
      timeout,
      parseItems,
      CAMLQuery,
      CAMLViewFields,
      CAMLQueryOptions,
      maxBatchSize,
      rawXml,
      viewColumns,
      debug,
    } = getListsOptions(options);

    let listViewThreshold;
    // Max Batch Size is truthy and less than list view threshold
    if (maxBatchSize && maxBatchSize < LIST_VIEW_THRESHOLD) {
      listViewThreshold = maxBatchSize;
    } else {
      listViewThreshold = LIST_VIEW_THRESHOLD - 1;
    }

    let batches = 0;

    // Get the last item ID
    getLastItemID({
      listName,
      webURL,
      async,
      timeout,
    })
      .then((lastItemID) => {
        batches = Math.ceil(lastItemID / listViewThreshold);
        return batchGetListItems({
          batches,
          listViewThreshold,
          listName,
          webURL,
          query: CAMLQuery,
          CAMLViewFields,
          CAMLQueryOptions,
          rawXml,
          timeout,
          debug,
        });
      })
      .then((rows) => {
        if (parseItems) {
          parseListItems({
            rows,
            CAMLViewFields,
            allowNullValues: false,
            viewColumns,
          }).then((items) => {
            resolve(items);
          });
        } else {
          resolve(rows);
        }
      })
      .catch((err) => {
        reject(err);
      });
  });
};

export default getThrottledListItems;
