import getThrottledListItems from "./getThrottledListItems";

/**
 * @name getThrottledMultiListItems
 * @description Returns information about items in multiple lists over list view threshold based on the same specified query per list.
 * @required listNames
 * @param {object} options
 * @param {Array} options.listNames - An array of list display names.
 * - Required: Yes
 * @param {string} [options.webURL=""] - The web site url
 * - Required: No
 * - Default: "" (current site)
 * @param {string} [options.CAMLViewFields="<ViewFields Properties='True' />"] - The view fields as an XML string
 * - Required: No
 * - Default: "<ViewFields Properties='True' />" (Returns all list fields)
 * @param {string} [options.CAMLQuery="<Query></Query>"] - The CAML query as an XML string
 * - Required: No
 * - Default: "<Query></Query>" (Returns all list fields)
 * @param {string} [options.CAMLQueryOptions="<QueryOptions></QueryOptions>"] - Additional query options as an XML string.
 * - Required: No
 * - Default: "<QueryOptions></QueryOptions>" (No query options)
 * @param {number} [options.CAMLRowLimit=0] - The number of rows to retrieve
 * - Required: No
 * - Default: 0 (No row limit (excluded list view threshold restraints))
 * @param {number} [options.timeout=0] - The timeout (pause) between batch requests.
 * - Required: No
 * - Default: 10 (ms)
 * @param {boolean} [options.async=true] - Synchronisation Type
 * - Required: No
 * - Default: true (asynchronous)
 * @param {boolean} [options.parseItems=false] - XML will be parsed with items return
 * - Required: No
 * - Default: false
 * @param {boolean} [options.viewColumns=null] - Only columns that are selectable in views will be returned (only if parsing is true)
 * - Required: No
 * - Default: null
 * @returns {Promise} Returns a Promise XML result.
 */
const getThrottledMultiListItems = (options) => {
  return new Promise((resolve, reject) => {
    // Ensure options.listNames is an array
    if (options.listNames && !Array.isArray(options.listNames))
      options.listNames = [options.listNames];

    // Store XML rows
    const rows = [];

    // Create Lists Array (used to iterate through each list)
    let lists = [...options.listNames];

    // Remove options.listNames
    delete options.listNames;

    // Error Counter to count caught errors
    // A rejection will occur if all loops result in an error
    let caughtErrors = 0;
    let allowedErrors = lists.length - 1;

    // Begin Iterating Through Each List
    (function loop() {
      // If all lists have been requested. End loop and resolve.
      if (!lists || lists.length === 0) {
        // Convert Multi Dimensional Array to 1 Dimension Array and Resolve;
        resolve([].concat(...rows));
        return;
      }

      // Get List Names from List Names Array
      options.listName = lists.shift();

      // Get List Items for List
      getThrottledListItems(options)
        .then((xml) => {
          // Push XML response to rows constant
          if (xml) {
            rows.push(xml);
          }

          // if timeout is defined
          if (options.timeout) {
            // Call loop after timeout
            setTimeout(() => {
              loop();
            }, options.timeout);
          } else {
            // No timeout defined
            loop();
          }
        })
        .catch((err) => {
          caughtErrors += 1;
          if (caughtErrors >= allowedErrors) {
            return reject(err);
          }

          // if timeout is defined
          if (options.timeout) {
            // Call loop after timeout
            setTimeout(() => {
              loop();
            }, options.timeout);
          } else {
            // No timeout defined
            loop();
          }
        });
    })();
  });
};

export default getThrottledMultiListItems;
