const viewColumnsArray = [
  "Attachments",
  "Author",
  "Created",
  "Editor",
  "ID",
  "Modified",
  "Title",
];
const listColumnsArray = [
  "_CopySource",
  "_EditMenuTableEnd",
  "_EditMenuTableStart",
  "_EditMenuTableStart2",
  "_HasCopyDestinations",
  "_IsCurrentVersion",
  "_Level",
  "_ModerationComments",
  "_ModerationStatus",
  "_UIVersion",
  "_UIVersionString",
  "Attachments",
  "Author",
  "BaseName",
  "ContentType",
  "ContentTypeId",
  "Created_x0020_Date",
  "Created",
  "DocIcon",
  "Edit",
  "Editor",
  "EncodedAbsUrl",
  "File_x0020_Type",
  "FileDirRef",
  "FileLeafRef",
  "FileRef",
  "FolderChildCount",
  "FSObjType",
  "GUID",
  "HTML_x0020_File_x0020_Type",
  "ID",
  "InstanceID",
  "ItemChildCount",
  "Last_x0020_Modified",
  "LinkFilename",
  "LinkFilename2",
  "LinkFilenameNoMenu",
  "LinkTitle",
  "LinkTitle2",
  "LinkTitleNoMenu",
  "MetaInfo",
  "Modified",
  "Order",
  "owshiddenversion",
  "PermMask",
  "ProgId",
  "ScopeId",
  "SelectTitle",
  "ServerUrl",
  "SortBehavior",
  "SyncClientId",
  "Title",
  "UniqueId",
  "WorkflowInstanceID",
  "WorkflowVersion",
];
/**
 * @name parseList
 * @description Returns an array of objects (items) from XML
 * @param {string} xml An XML string of list data.
 * @param {Array} [columnAttributes=[]] An array of strings for attributes to be returned.
 * @param {Boolean} [customColumns=null] If true, only custom columns are returned.
 * @param {Boolean} [hiddenColumns=null] If true, only hidden columns are returned.
 * @param {Boolean} [readOnlyColumns=null] If true, only read only columns are returned.
 * @param {Boolean} [viewColumns=null] If true, only standard view columns are returned.
 * @returns {Promise} Returns a Promise array of objects result.
 */
const parseList = ({
  xml,
  columnAttributes = [],
  customColumns = null,
  hiddenColumns = null,
  readOnlyColumns = null,
  viewColumns = null,
} = {}) => {
  return new Promise((resolve, reject) => {
    // Create list object
    let list = [...xml.querySelector("List").attributes].reduce(
      (object, { nodeName, nodeValue }) => {
        object[nodeName] = nodeValue;
        return object;
      },
      {}
    );

    // Create list fields object
    list.fields = [...xml.querySelectorAll("Fields > Field")]
      .filter((field) => {
        const isHidden = field.getAttribute("Hidden") === "TRUE";
        const isReadOnly = field.getAttribute("ReadOnly") === "TRUE";
        const staticName = field.getAttribute("StaticName");
        const isCustom = listColumnsArray.indexOf(staticName) === -1;
        const isViewColumn = viewColumnsArray.indexOf(staticName) === -1;

        // Validate custom columns
        if (
          (customColumns === true && !isCustom) ||
          (customColumns === false && isCustom)
        ) {
          return false;
        }

        // Validate Read Only Columns
        if (
          (readOnlyColumns === true && !isReadOnly) ||
          (readOnlyColumns === false && isReadOnly)
        ) {
          return false;
        }

        // Validate Hidden Columns
        if (
          (hiddenColumns === true && !isHidden) ||
          (hiddenColumns === false && isHidden)
        ) {
          return false;
        }

        // Validate View Columns
        if (
          (viewColumns === true && isViewColumn && !isCustom) ||
          (viewColumns === false && !isViewColumn && !isCustom)
        ) {
          return false;
        }

        // Default of return true
        return true;
      })
      .map((node) => {
        let field = [...node.attributes].reduce(
          (object, { nodeName, nodeValue }) => {
            // If columns attributes options are supplied and the attribute is not required, skip
            if (
              columnAttributes.length > 0 &&
              !columnAttributes.includes(nodeName)
            )
              return object;

            // Assign value to object
            object[nodeName] = nodeValue;

            // Return object
            return object;
          },
          {}
        );

        // Set default value
        field.Default = (node.querySelector("Default") || {}).textContent || "";

        // Switch by field type
        switch (field.Type) {
          case "Choice":
          case "MultiChoice":
            field.Choices = [...node.querySelectorAll("CHOICE")].map(
              ({ textContent }) => textContent
            );
            break;
          case "Calculated":
            // Formula
            field.Formula =
              (node.querySelector("Formula") || {}).textContent || "";

            // Formula Display Names
            field.FormulaDisplayNames =
              (node.querySelector("FormulaDisplayNames") || {}).textContent ||
              "";

            // Formula Field Refs
            field.FieldRefs = [...node.querySelectorAll("FieldRef")].map(
              (node) => node.getAttribute("Name") || ""
            );
            break;

          default:
            break;
        }

        // Return field object
        return field;
      });

    // Resolve list object (with field data)
    resolve(list);
  });
};

export default parseList;
